import pymysql

from silence.settings import settings

###############################################################################
# The connector fetches the relevant configuration parameters
# and uses them to build a connection to the database.
###############################################################################

global conn
conn = None

def get_conn():
    # Only connect lazily
    global conn
    if conn is None:
        conn = pymysql.connect(
        host=settings.DB_CONN["host"],
        port=settings.DB_CONN["port"],
        user=settings.DB_CONN["username"],
        password=settings.DB_CONN["password"],
        database=settings.DB_CONN["database"]
    )
    return conn




###############################################################################
# Default settings, which can be overriden by each individual project.
###############################################################################

DEBUG_ENABLED = True
HTTP_PORT = 8080
API_PREFIX = ""

DB_CONN = {
    "host": "localhost",
    "port": 3306,
    "username": "default_username",
    "password": "default_password",
    "database": "default_database",
}

SECRET_KEY = "These are generated automatically for each project."

SQL_SCRIPTS = []

PROJECT_TEMPLATE_NAME = "silence-app"
PROJECT_TEMPLATE_REPO = "https://github.com/agu-borrego/Silence-project-template.git"



from pymysql.cursors import DictCursor

from silence.db.connector import get_conn
from silence.exceptions import DatabaseError

###############################################################################
# The DAL (Data Access Layer) functions provide an abstraction layer
# for querying and updating the database.
###############################################################################

# Query method to retrieve information
def query(q, params=None):
    # Fetch the connection and get a cursor
    conn = get_conn()
    cursor = conn.cursor(DictCursor)
    try:
        # Execute the query, with or without parameters and return the result
        if params:
            cursor.execute(q, params)
        else:
            cursor.execute(q)

        res = cursor.fetchall()
        return res
    except Exception as exc:
        # If anything happens, wrap the exceptions in a DatabaseError
        raise DatabaseError(exc) from exc
    finally:
        # Close the cursor
        cursor.close()


# Update method to modify information
def update(q, params=None):
    conn = get_conn()
    cursor = conn.cursor(DictCursor)

    # Fetch the connection and get a cursor
    try:
        # Execute the query, with or without parameters and return the result
        if params:
            cursor.execute(q, params)
        else:
            cursor.execute(q)

        conn.commit()

        # Return the ID of the row that was modified or inserted
        return cursor.lastrowid
    except Exception as exc:
        # If anything happens, wrap the exceptions in a DatabaseError
        raise DatabaseError(exc) from exc
    finally:
        # Close the cursor
        cursor.close()


from flask import Flask

from silence.server.endpoint_loader import load_user_endpoints
from silence.settings import settings

###############################################################################
# The server manager is responsible for setting up the Flask webserver,
# configuring it and deploying the endpoints and web app.
###############################################################################

APP = Flask(__name__)

def setup():
    # Configures the web server
    APP.secret_key = settings.SECRET_KEY 
    APP.config["SESSION_TYPE"] = "filesystem"

    load_user_endpoints()

    # TO-DO remove this later
    @APP.route("/test")
    def test():
        return "Hola! :D"


def run():
    
    APP.run(
        port=settings.HTTP_PORT,
        debug=settings.DEBUG_ENABLED,
    )
    


from functools import wraps
from flask import jsonify

from silence.server import manager as server_manager
from silence.db.dal import query
from silence.settings import settings

###############################################################################
# This is where the fun at
###############################################################################

def endpoint(route, method, sql, auth_required=False):
    route_prefix = settings.API_PREFIX
    if route_prefix.endswith("/"):
        route_prefix = route_prefix[:-1]  # Drop the final /

    def wrapper(func):
        @wraps(func)
        def decorator(*args, **kwargs):
            func(*args, **kwargs)

        def route_handler():
            decorator()
            res = query(sql)
            return jsonify(res), 200
        
        server_manager.APP.add_url_rule(route_prefix + route, route, route_handler, methods=[method])

        return decorator
    return wrapper


import importlib
import inspect

from os import listdir
from os.path import splitext

###############################################################################
# Look for .py files inside the project's "/api" folder
# and force them to run, activating the @endpoint decorators
###############################################################################

def load_user_endpoints():
    print("Loading user endpoints...")
    
    # Load every .py file inside the api/ folder
    for pyfile in listdir("api"):
        module_name = "api." + splitext(pyfile)[0]

        try:
            importlib.import_module(module_name)
        except ImportError:
            raise RuntimeError(f"Could not load the API file {module_name}")
    